<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\User\Login;

// Import framework stuff
use Org\Mxchange\CoreFramework\Bootstrap\FrameworkBootstrap;
use Org\Mxchange\CoreFramework\Database\Frontend\User\UserDatabaseFrontend;
use Org\Mxchange\CoreFramework\Factory\Object\ObjectFactory;
use Org\Mxchange\CoreFramework\Manager\Login\ManageableMember;
use Org\Mxchange\CoreFramework\Registry\Registerable;
use Org\Mxchange\CoreFramework\User\BaseUser;
use Org\Mxchange\CoreFramework\User\UsernameMissingException;

/**
 * A generic class for handling users
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2022 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class Member extends BaseUser implements ManageableMember, Registerable {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	private function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this user class by a provided username. This
	 * factory method will check if username is already taken and if not so it
	 * will throw an exception.
	 *
	 * @param	$userName		Username we need a class instance for
	 * @return	$userInstance	An instance of this user class
	 * @throws	UsernameMissingException	If the username does not exist
	 * @throws	UnexpectedGuestAccountException		If the user status is 'guest'
	 */
	public static final function createMemberByUsername ($userName) {
		// Get a new instance
		$userInstance = new Member();

		// Set the username
		$userInstance->setUserName($userName);

		// Check if username exists
		if ($userInstance->ifUsernameExists() === false) {
			// Throw an exception here
			throw new UsernameMissingException(array($userInstance, $userName), self::EXCEPTION_USERNAME_NOT_FOUND);
		} elseif ($userInstance->isGuest() === true) {
			// User should not be a guest here
			throw new UnexpectedGuestAccountException(array($userInstance, $userName), self::EXCEPTION_USER_IS_GUEST);
		}

		// Return the instance
		return $userInstance;
	}

	/**
	 * Creates an instance of this user class by a provided email address. This
	 * factory method will not check if email address is there.
	 *
	 * @param	$email			Email address of the user
	 * @return	$userInstance	An instance of this user class
	 */
	public static final function createMemberByEmail ($email) {
		// Get a new instance
		$userInstance = new Member();

		// Set the username
		$userInstance->setEmail($email);

		// Return the instance
		return $userInstance;
	}

	/**
	 * Creates a user by a given request instance
	 *
	 * @param	$requestInstance	An instance of a Requestable class
	 * @return	$userInstance		An instance of this user class
	 * @todo	Add more ways over creating user classes
	 */
	public static final function createMemberByRequest (Requestable $requestInstance) {
		// Determine if by email or username
		if (!is_null($requestInstance->getRequestElement('username'))) {
			// Username supplied
			$userInstance = self::createMemberByUserName($requestInstance->getRequestElement('username'));
		} elseif (!is_null($requestInstance->getRequestElement('email'))) {
			// Email supplied
			$userInstance = self::createMemberByEmail($requestInstance->getRequestElement('email'));
		} else {
			// Unsupported mode
			$userInstance = new Member();
			$userInstance->debugBackTrace('More ways of creating user classes are needed here.');
			exit();
		}

		// Return the prepared instance
		return $userInstance;
	}

	/**
	 * Updates the last activity timestamp and last performed action in the
	 * database result.
	 *
	 * @param	$requestInstance	A requestable class instance
	 * @return	void
	 */
	public function updateLastActivity (Requestable $requestInstance) {
		// Set last action
		$lastAction = $requestInstance->getRequestElement('action');

		// If there is no action use the default on
		if (is_null($lastAction)) {
			$lastAction = FrameworkBootstrap::getConfigurationInstance()->getConfigEntry('login_default_action');
		}

		// Get a critieria instance
		$searchInstance = ObjectFactory::createObjectByConfiguredName('search_criteria_class');

		// Add search criteria
		$searchInstance->addCriteria(UserDatabaseFrontend::DB_COLUMN_USERNAME, $this->getUserName());
		$searchInstance->setLimit(1);

		// Now get another criteria
		$updateInstance = ObjectFactory::createObjectByConfiguredName('update_criteria_class');

		// And add our both entries
		$updateInstance->addCriteria('last_activity', date('Y-m-d H:i:s', time()));
		$updateInstance->addCriteria('last_action', $lastAction);

		// Add the search criteria for searching for the right entry
		$updateInstance->setSearchInstance($searchInstance);

		// Set frontend class name
		$updateInstance->setFrontendConfigEntry('user_db_frontend_class');

		// Remember the update in database result
		$this->getResultInstance()->add2UpdateQueue($updateInstance);
	}

}
